# This file was auto-generated by Fern from our API Definition.

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .resources.v_1.client import AsyncV1Client, V1Client


class NonInsurancePayersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper
        self.v_1 = V1Client(client_wrapper=self._client_wrapper)


class AsyncNonInsurancePayersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper
        self.v_1 = AsyncV1Client(client_wrapper=self._client_wrapper)



# This file was auto-generated by Fern from our API Definition.

from .bad_request_error import BadRequestError
from .not_found_error import NotFoundError
from .version_conflict_error import VersionConflictError

__all__ = ["BadRequestError", "NotFoundError", "VersionConflictError"]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import datetime as dt
import typing

import pydantic

from ....core.datetime_utils import serialize_datetime
from ....core.pydantic_utilities import deep_union_pydantic_dicts
from ...commons.types.appointment_id import AppointmentId
from ...commons.types.claim_id import ClaimId
from ...commons.types.encounter_id import EncounterId
from ...commons.types.patient_external_id import PatientExternalId
from ...commons.types.provider_id import ProviderId
from ...commons.types.service_line_id import ServiceLineId


class AllocationTarget_ServiceLine(pydantic.BaseModel):
    """
    Allocation targets describe whether the portion of a payment is being applied toward a specific service line,
    claim, billing provider, or is unallocated.
    """

    service_line_id: ServiceLineId
    claim_id: ClaimId
    encounter_id: EncounterId
    type: typing.Literal["service_line"] = "service_line"

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}


class AllocationTarget_Claim(pydantic.BaseModel):
    """
    Allocation targets describe whether the portion of a payment is being applied toward a specific service line,
    claim, billing provider, or is unallocated.
    """

    claim_id: ClaimId
    encounter_id: EncounterId
    type: typing.Literal["claim"] = "claim"

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}


class AllocationTarget_BillingProviderId(pydantic.BaseModel):
    """
    Allocation targets describe whether the portion of a payment is being applied toward a specific service line,
    claim, billing provider, or is unallocated.
    """

    billing_provider_id: ProviderId
    type: typing.Literal["billing_provider_id"] = "billing_provider_id"

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}


class AllocationTarget_Appointment(pydantic.BaseModel):
    """
    Allocation targets describe whether the portion of a payment is being applied toward a specific service line,
    claim, billing provider, or is unallocated.
    """

    appointment_id: AppointmentId
    patient_external_id: PatientExternalId
    type: typing.Literal["appointment"] = "appointment"

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}


class AllocationTarget_Unattributed(pydantic.BaseModel):
    """
    Allocation targets describe whether the portion of a payment is being applied toward a specific service line,
    claim, billing provider, or is unallocated.
    """

    type: typing.Literal["unattributed"] = "unattributed"

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}


AllocationTarget = typing.Union[
    AllocationTarget_ServiceLine,
    AllocationTarget_Claim,
    AllocationTarget_BillingProviderId,
    AllocationTarget_Appointment,
    AllocationTarget_Unattributed,
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ........core.datetime_utils import serialize_datetime
from ........core.pydantic_utilities import deep_union_pydantic_dicts
from .....appointments.resources.v_1.types.mutable_appointment import MutableAppointment
from .....coverages.resources.v_1.types.mutable_coverage import MutableCoverage
from .....patients.resources.v_1.types.patient import Patient


class PatientListItem(pydantic.BaseModel):
    patient: Patient
    primary_coverage: typing.Optional[MutableCoverage] = None
    next_appointment: typing.Optional[MutableAppointment] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ......core.datetime_utils import serialize_datetime
from ......core.pydantic_utilities import deep_union_pydantic_dicts
from .intake_question_id import IntakeQuestionId
from .intake_response_and_follow_ups import IntakeResponseAndFollowUps


class IntakeQuestion(pydantic.BaseModel):
    """
    Examples
    --------
    from candid.resources.encounters.v_4 import (
        IntakeFollowUp,
        IntakeQuestion,
        IntakeResponseAndFollowUps,
    )

    IntakeQuestion(
        id="6E7FBCE4-A8EA-46D0-A8D8-FF83CA3BB176",
        text="Do you have any allergies?",
        responses=[
            IntakeResponseAndFollowUps(
                response="No allergies",
                follow_ups=[
                    IntakeFollowUp(
                        id="4F3D57F9-AC94-49D6-87E4-E804B709917A",
                        text="Do you have any allergies?",
                        response="No allergies",
                    )
                ],
            )
        ],
    )
    """

    id: IntakeQuestionId
    text: str
    responses: typing.Optional[typing.List[IntakeResponseAndFollowUps]] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ....core.datetime_utils import serialize_datetime
from ....core.pydantic_utilities import deep_union_pydantic_dicts
from ...commons.types.street_address_long_zip import StreetAddressLongZip
from .service_facility_id import ServiceFacilityId


class EncounterServiceFacility(pydantic.BaseModel):
    """
    Examples
    --------
    import uuid

    from candid import EncounterServiceFacility, State, StreetAddressLongZip

    EncounterServiceFacility(
        service_facility_id=uuid.UUID(
            "2861487b-232c-4ded-a874-616a5db0688f",
        ),
        organization_name="Test Organization",
        address=StreetAddressLongZip(
            address_1="123 Main St",
            address_2="Apt 1",
            city="New York",
            state=State.NY,
            zip_code="10001",
            zip_plus_four_code="1234",
        ),
    )
    """

    service_facility_id: ServiceFacilityId
    organization_name: str
    npi: typing.Optional[str] = pydantic.Field(default=None)
    """
    An NPI specific to the service facility if applicable, i.e. if it has one and is not under the billing provider's NPI.
    Box 32 section (a) of the CMS-1500 claim form.
    """

    address: StreetAddressLongZip = pydantic.Field()
    """
    zip_plus_four_code is required for service facility address. When the zip_plus_four_code is not available use "9998" as per CMS documentation.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .third_party_payer_refund import ThirdPartyPayerRefund
from .third_party_payer_refund_create import ThirdPartyPayerRefundCreate
from .third_party_payer_refund_id import ThirdPartyPayerRefundId
from .third_party_payer_refund_sort_field import ThirdPartyPayerRefundSortField
from .third_party_payer_refunds_page import ThirdPartyPayerRefundsPage

__all__ = [
    "ThirdPartyPayerRefund",
    "ThirdPartyPayerRefundCreate",
    "ThirdPartyPayerRefundId",
    "ThirdPartyPayerRefundSortField",
    "ThirdPartyPayerRefundsPage",
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ExternalProviderType(str, enum.Enum):
    PRIMARY = "PRIMARY"
    REFERRING = "REFERRING"
    ATTENDING = "ATTENDING"

    def visit(
        self,
        primary: typing.Callable[[], T_Result],
        referring: typing.Callable[[], T_Result],
        attending: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ExternalProviderType.PRIMARY:
            return primary()
        if self is ExternalProviderType.REFERRING:
            return referring()
        if self is ExternalProviderType.ATTENDING:
            return attending()



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.jsonable_encoder import jsonable_encoder
from .....core.pydantic_utilities import pydantic_v1
from .....core.request_options import RequestOptions
from ....commons.errors.entity_conflict_error import EntityConflictError
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.errors.unprocessable_entity_error import UnprocessableEntityError
from ....commons.types.entity_conflict_error_message import EntityConflictErrorMessage
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from ....commons.types.page_token import PageToken
from ....commons.types.sort_direction import SortDirection
from ....commons.types.unprocessable_entity_error_message import UnprocessableEntityErrorMessage
from .types.create_non_insurance_payer_request import CreateNonInsurancePayerRequest
from .types.non_insurance_payer import NonInsurancePayer
from .types.non_insurance_payer_id import NonInsurancePayerId
from .types.non_insurance_payer_page import NonInsurancePayerPage
from .types.non_insurance_payer_sort_field import NonInsurancePayerSortField
from .types.non_insurance_payer_update_request import NonInsurancePayerUpdateRequest
from .types.toggle_non_insurance_payer_enablement_request import ToggleNonInsurancePayerEnablementRequest

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self, *, request: CreateNonInsurancePayerRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> NonInsurancePayer:
        """
        Parameters
        ----------
        request : CreateNonInsurancePayerRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        NonInsurancePayer

        Examples
        --------
        from candid.client import CandidApiClient
        from candid.resources.non_insurance_payers.v_1 import (
            CreateNonInsurancePayerRequest,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.non_insurance_payers.v_1.create(
            request=CreateNonInsurancePayerRequest(
                name="string",
                description="string",
                category="string",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/non-insurance-payers/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(NonInsurancePayer, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "EntityConflictError":
                raise EntityConflictError(
                    pydantic_v1.parse_obj_as(EntityConflictErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def toggle_enablement(
        self,
        non_insurance_payer_id: NonInsurancePayerId,
        *,
        request: ToggleNonInsurancePayerEnablementRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> NonInsurancePayer:
        """
        Parameters
        ----------
        non_insurance_payer_id : NonInsurancePayerId

        request : ToggleNonInsurancePayerEnablementRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        NonInsurancePayer

        Examples
        --------
        import uuid

        from candid.client import CandidApiClient
        from candid.resources.non_insurance_payers.v_1 import (
            ToggleNonInsurancePayerEnablementRequest,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.non_insurance_payers.v_1.toggle_enablement(
            non_insurance_payer_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            request=ToggleNonInsurancePayerEnablementRequest(
                enabled=True,
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/non-insurance-payers/v1/{jsonable_encoder(non_insurance_payer_id)}/toggle_enablement",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(NonInsurancePayer, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_multi(
        self,
        *,
        name: typing.Optional[str] = None,
        category: typing.Optional[str] = None,
        enabled: typing.Optional[bool] = None,
        sort: typing.Optional[NonInsurancePayerSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        limit: typing.Optional[int] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> NonInsurancePayerPage:
        """
        Parameters
        ----------
        name : typing.Optional[str]

        category : typing.Optional[str]

        enabled : typing.Optional[bool]

        sort : typing.Optional[NonInsurancePayerSortField]

        sort_direction : typing.Optional[SortDirection]

        limit : typing.Optional[int]
            Defaults to 100

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        NonInsurancePayerPage

        Examples
        --------
        from candid import SortDirection
        from candid.client import CandidApiClient
        from candid.resources.non_insurance_payers.v_1 import NonInsurancePayerSortField

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.non_insurance_payers.v_1.get_multi(
            name="string",
            category="string",
            enabled=True,
            sort=NonInsurancePayerSortField.NAME,
            sort_direction=SortDirection.ASC,
            limit=1,
            page_token="eyJ0b2tlbiI6IjEiLCJwYWdlX3Rva2VuIjoiMiJ9",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/non-insurance-payers/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "name": name,
                "category": category,
                "enabled": enabled,
                "sort": sort,
                "sort_direction": sort_direction,
                "limit": limit,
                "page_token": page_token,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(NonInsurancePayerPage, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(UnprocessableEntityErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, non_insurance_payer_id: NonInsurancePayerId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> NonInsurancePayer:
        """
        Parameters
        ----------
        non_insurance_payer_id : NonInsurancePayerId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        NonInsurancePayer

        Examples
        --------
        import uuid

        from candid.client import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.non_insurance_payers.v_1.get(
            non_insurance_payer_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/non-insurance-payers/v1/{jsonable_encoder(non_insurance_payer_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(NonInsurancePayer, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        non_insurance_payer_id: NonInsurancePayerId,
        *,
        request: NonInsurancePayerUpdateRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> NonInsurancePayer:
        """
        Parameters
        ----------
        non_insurance_payer_id : NonInsurancePayerId

        request : NonInsurancePayerUpdateRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        NonInsurancePayer

        Examples
        --------
        import uuid

        from candid.client import CandidApiClient
        from candid.resources.non_insurance_payers.v_1 import (
            NonInsurancePayerCategoryUpdate,
            NonInsurancePayerDescriptionUpdate,
            NonInsurancePayerUpdateRequest,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.non_insurance_payers.v_1.update(
            non_insurance_payer_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            request=NonInsurancePayerUpdateRequest(
                name="string",
                description=NonInsurancePayerDescriptionUpdate(),
                category=NonInsurancePayerCategoryUpdate(),
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/non-insurance-payers/v1/{jsonable_encoder(non_insurance_payer_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(NonInsurancePayer, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "EntityConflictError":
                raise EntityConflictError(
                    pydantic_v1.parse_obj_as(EntityConflictErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self, non_insurance_payer_id: NonInsurancePayerId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Parameters
        ----------
        non_insurance_payer_id : NonInsurancePayerId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import uuid

        from candid.client import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.non_insurance_payers.v_1.delete(
            non_insurance_payer_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/non-insurance-payers/v1/{jsonable_encoder(non_insurance_payer_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self, *, request: CreateNonInsurancePayerRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> NonInsurancePayer:
        """
        Parameters
        ----------
        request : CreateNonInsurancePayerRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        NonInsurancePayer

        Examples
        --------
        import asyncio

        from candid.client import AsyncCandidApiClient
        from candid.resources.non_insurance_payers.v_1 import (
            CreateNonInsurancePayerRequest,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.non_insurance_payers.v_1.create(
                request=CreateNonInsurancePayerRequest(
                    name="string",
                    description="string",
                    category="string",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/non-insurance-payers/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(NonInsurancePayer, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "EntityConflictError":
                raise EntityConflictError(
                    pydantic_v1.parse_obj_as(EntityConflictErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def toggle_enablement(
        self,
        non_insurance_payer_id: NonInsurancePayerId,
        *,
        request: ToggleNonInsurancePayerEnablementRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> NonInsurancePayer:
        """
        Parameters
        ----------
        non_insurance_payer_id : NonInsurancePayerId

        request : ToggleNonInsurancePayerEnablementRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        NonInsurancePayer

        Examples
        --------
        import asyncio
        import uuid

        from candid.client import AsyncCandidApiClient
        from candid.resources.non_insurance_payers.v_1 import (
            ToggleNonInsurancePayerEnablementRequest,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.non_insurance_payers.v_1.toggle_enablement(
                non_insurance_payer_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                request=ToggleNonInsurancePayerEnablementRequest(
                    enabled=True,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/non-insurance-payers/v1/{jsonable_encoder(non_insurance_payer_id)}/toggle_enablement",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(NonInsurancePayer, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_multi(
        self,
        *,
        name: typing.Optional[str] = None,
        category: typing.Optional[str] = None,
        enabled: typing.Optional[bool] = None,
        sort: typing.Optional[NonInsurancePayerSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        limit: typing.Optional[int] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> NonInsurancePayerPage:
        """
        Parameters
        ----------
        name : typing.Optional[str]

        category : typing.Optional[str]

        enabled : typing.Optional[bool]

        sort : typing.Optional[NonInsurancePayerSortField]

        sort_direction : typing.Optional[SortDirection]

        limit : typing.Optional[int]
            Defaults to 100

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        NonInsurancePayerPage

        Examples
        --------
        import asyncio

        from candid import SortDirection
        from candid.client import AsyncCandidApiClient
        from candid.resources.non_insurance_payers.v_1 import NonInsurancePayerSortField

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.non_insurance_payers.v_1.get_multi(
                name="string",
                category="string",
                enabled=True,
                sort=NonInsurancePayerSortField.NAME,
                sort_direction=SortDirection.ASC,
                limit=1,
                page_token="eyJ0b2tlbiI6IjEiLCJwYWdlX3Rva2VuIjoiMiJ9",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/non-insurance-payers/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "name": name,
                "category": category,
                "enabled": enabled,
                "sort": sort,
                "sort_direction": sort_direction,
                "limit": limit,
                "page_token": page_token,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(NonInsurancePayerPage, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(UnprocessableEntityErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, non_insurance_payer_id: NonInsurancePayerId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> NonInsurancePayer:
        """
        Parameters
        ----------
        non_insurance_payer_id : NonInsurancePayerId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        NonInsurancePayer

        Examples
        --------
        import asyncio
        import uuid

        from candid.client import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.non_insurance_payers.v_1.get(
                non_insurance_payer_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/non-insurance-payers/v1/{jsonable_encoder(non_insurance_payer_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(NonInsurancePayer, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        non_insurance_payer_id: NonInsurancePayerId,
        *,
        request: NonInsurancePayerUpdateRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> NonInsurancePayer:
        """
        Parameters
        ----------
        non_insurance_payer_id : NonInsurancePayerId

        request : NonInsurancePayerUpdateRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        NonInsurancePayer

        Examples
        --------
        import asyncio
        import uuid

        from candid.client import AsyncCandidApiClient
        from candid.resources.non_insurance_payers.v_1 import (
            NonInsurancePayerCategoryUpdate,
            NonInsurancePayerDescriptionUpdate,
            NonInsurancePayerUpdateRequest,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.non_insurance_payers.v_1.update(
                non_insurance_payer_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                request=NonInsurancePayerUpdateRequest(
                    name="string",
                    description=NonInsurancePayerDescriptionUpdate(),
                    category=NonInsurancePayerCategoryUpdate(),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/non-insurance-payers/v1/{jsonable_encoder(non_insurance_payer_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(NonInsurancePayer, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "EntityConflictError":
                raise EntityConflictError(
                    pydantic_v1.parse_obj_as(EntityConflictErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, non_insurance_payer_id: NonInsurancePayerId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Parameters
        ----------
        non_insurance_payer_id : NonInsurancePayerId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio
        import uuid

        from candid.client import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.non_insurance_payers.v_1.delete(
                non_insurance_payer_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/non-insurance-payers/v1/{jsonable_encoder(non_insurance_payer_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class PatientRefundSortField(str, enum.Enum):
    REFUND_SOURCE = "refund_source"
    AMOUNT_CENTS = "amount_cents"
    REFUND_TIMESTAMP = "refund_timestamp"
    REFUND_REASON = "refund_reason"
    REFUND_NOTE = "refund_note"

    def visit(
        self,
        refund_source: typing.Callable[[], T_Result],
        amount_cents: typing.Callable[[], T_Result],
        refund_timestamp: typing.Callable[[], T_Result],
        refund_reason: typing.Callable[[], T_Result],
        refund_note: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is PatientRefundSortField.REFUND_SOURCE:
            return refund_source()
        if self is PatientRefundSortField.AMOUNT_CENTS:
            return amount_cents()
        if self is PatientRefundSortField.REFUND_TIMESTAMP:
            return refund_timestamp()
        if self is PatientRefundSortField.REFUND_REASON:
            return refund_reason()
        if self is PatientRefundSortField.REFUND_NOTE:
            return refund_note()



# This file was auto-generated by Fern from our API Definition.

from .create_third_party_payer_request import CreateThirdPartyPayerRequest
from .third_party_payer import ThirdPartyPayer
from .third_party_payer_category_update import (
    ThirdPartyPayerCategoryUpdate,
    ThirdPartyPayerCategoryUpdate_Remove,
    ThirdPartyPayerCategoryUpdate_Set,
)
from .third_party_payer_description_update import (
    ThirdPartyPayerDescriptionUpdate,
    ThirdPartyPayerDescriptionUpdate_Remove,
    ThirdPartyPayerDescriptionUpdate_Set,
)
from .third_party_payer_id import ThirdPartyPayerId
from .third_party_payer_page import ThirdPartyPayerPage
from .third_party_payer_sort_field import ThirdPartyPayerSortField
from .third_party_payer_update_request import ThirdPartyPayerUpdateRequest
from .toggle_third_party_payer_enablement_request import ToggleThirdPartyPayerEnablementRequest

__all__ = [
    "CreateThirdPartyPayerRequest",
    "ThirdPartyPayer",
    "ThirdPartyPayerCategoryUpdate",
    "ThirdPartyPayerCategoryUpdate_Remove",
    "ThirdPartyPayerCategoryUpdate_Set",
    "ThirdPartyPayerDescriptionUpdate",
    "ThirdPartyPayerDescriptionUpdate_Remove",
    "ThirdPartyPayerDescriptionUpdate_Set",
    "ThirdPartyPayerId",
    "ThirdPartyPayerPage",
    "ThirdPartyPayerSortField",
    "ThirdPartyPayerUpdateRequest",
    "ToggleThirdPartyPayerEnablementRequest",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ......core.datetime_utils import serialize_datetime
from ......core.pydantic_utilities import deep_union_pydantic_dicts
from .....commons.types.date import Date
from .....commons.types.regions import Regions
from .authorized_signatory import AuthorizedSignatory
from .contract_status import ContractStatus
from .insurance_types import InsuranceTypes


class ContractBase(pydantic.BaseModel):
    effective_date: Date = pydantic.Field()
    """
    The starting day upon which the contract is effective
    """

    expiration_date: typing.Optional[Date] = pydantic.Field(default=None)
    """
    An optional end day upon which the contract expires
    """

    regions: Regions = pydantic.Field()
    """
    The state(s) to which the contract's coverage extends.
    It may also be set to "national" for the entirety of the US.
    """

    contract_status: typing.Optional[ContractStatus] = None
    authorized_signatory: typing.Optional[AuthorizedSignatory] = None
    commercial_insurance_types: InsuranceTypes = pydantic.Field()
    """
    The commercial plan insurance types this contract applies.
    """

    medicare_insurance_types: InsuranceTypes = pydantic.Field()
    """
    The Medicare plan insurance types this contract applies.
    """

    medicaid_insurance_types: InsuranceTypes = pydantic.Field()
    """
    The Medicaid plan insurance types this contract applies.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class InterventionCategory(str, enum.Enum):
    ALLOPATHIC = "allopathic"
    NATUROPATHIC = "naturopathic"
    TESTS = "tests"
    LIFESTYLE = "lifestyle"

    def visit(
        self,
        allopathic: typing.Callable[[], T_Result],
        naturopathic: typing.Callable[[], T_Result],
        tests: typing.Callable[[], T_Result],
        lifestyle: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is InterventionCategory.ALLOPATHIC:
            return allopathic()
        if self is InterventionCategory.NATUROPATHIC:
            return naturopathic()
        if self is InterventionCategory.TESTS:
            return tests()
        if self is InterventionCategory.LIFESTYLE:
            return lifestyle()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class Gender(str, enum.Enum):
    MAN = "MAN"
    WOMAN = "WOMAN"
    NON_BINARY = "NON_BINARY"
    TWO_SPIRIT = "TWO_SPIRIT"
    OTHER = "OTHER"
    UNKNOWN = "UNKNOWN"
    REFUSED = "REFUSED"

    def visit(
        self,
        man: typing.Callable[[], T_Result],
        woman: typing.Callable[[], T_Result],
        non_binary: typing.Callable[[], T_Result],
        two_spirit: typing.Callable[[], T_Result],
        other: typing.Callable[[], T_Result],
        unknown: typing.Callable[[], T_Result],
        refused: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is Gender.MAN:
            return man()
        if self is Gender.WOMAN:
            return woman()
        if self is Gender.NON_BINARY:
            return non_binary()
        if self is Gender.TWO_SPIRIT:
            return two_spirit()
        if self is Gender.OTHER:
            return other()
        if self is Gender.UNKNOWN:
            return unknown()
        if self is Gender.REFUSED:
            return refused()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ......core.datetime_utils import serialize_datetime
from ......core.pydantic_utilities import deep_union_pydantic_dicts
from .....commons.types.patient_external_id import PatientExternalId
from .....invoices.resources.v_2.types.invoice_item_create import InvoiceItemCreate
from .....invoices.resources.v_2.types.invoice_status import InvoiceStatus
from .....payment_account_configs.types.payment_account_config_id import PaymentAccountConfigId


class CreateImportInvoiceRequest(pydantic.BaseModel):
    external_payment_account_config_id: PaymentAccountConfigId
    patient_external_id: PatientExternalId
    external_customer_identifier: str = pydantic.Field()
    """
    Id of the customer in the source system
    """

    note: typing.Optional[str] = None
    due_date: typing.Optional[dt.date] = pydantic.Field(default=None)
    """
    If given as None, days_until_due in the payment config will be used to create a default date
    """

    items: typing.List[InvoiceItemCreate]
    status: InvoiceStatus
    external_identifier: str = pydantic.Field()
    """
    Id of the invoice being imported in the source system. Warning - This field CANNOT be updated.
    """

    customer_invoice_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    Link to the patient view of the invoice in the third-party service
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class LineOfBusiness(str, enum.Enum):
    MEDICARE = "medicare"
    MEDICAID = "medicaid"
    COMMERCIAL = "commercial"

    def visit(
        self,
        medicare: typing.Callable[[], T_Result],
        medicaid: typing.Callable[[], T_Result],
        commercial: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is LineOfBusiness.MEDICARE:
            return medicare()
        if self is LineOfBusiness.MEDICAID:
            return medicaid()
        if self is LineOfBusiness.COMMERCIAL:
            return commercial()



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .......core.api_error import ApiError
from .......core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .......core.pydantic_utilities import pydantic_v1
from .......core.request_options import RequestOptions
from ....common.errors.bad_request_error import BadRequestError
from ....common.types.error_base import ErrorBase
from ....common.types.page_token import PageToken
from ....common.types.sort_direction import SortDirection
from .types.appointment_list_page import AppointmentListPage
from .types.filter_query_string import FilterQueryString
from .types.patient_list_page import PatientListPage


class V1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_patient_list(
        self,
        *,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        sort_field: typing.Optional[str] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        filters: typing.Optional[FilterQueryString] = None,
        request_options: typing.Optional[RequestOptions] = None
    ) -> PatientListPage:
        """
        Gets patients with dependent objects for patients that match the query parameters.

        Parameters
        ----------
        page_token : typing.Optional[PageToken]

        limit : typing.Optional[int]

        sort_field : typing.Optional[str]

        sort_direction : typing.Optional[SortDirection]
            Defaults to ascending.

        filters : typing.Optional[FilterQueryString]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatientListPage

        Examples
        --------
        from candid.client import CandidApiClient
        from candid.resources.pre_encounter import SortDirection

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.lists.v_1.get_patient_list(
            page_token="string",
            limit=1,
            sort_field="string",
            sort_direction=SortDirection.ASC,
            filters="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "lists/v1/patient",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "page_token": page_token,
                "limit": limit,
                "sort_field": sort_field,
                "sort_direction": sort_direction,
                "filters": filters,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(PatientListPage, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequestError":
                raise BadRequestError(pydantic_v1.parse_obj_as(ErrorBase, _response_json["content"]))  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_appointment_list(
        self,
        *,
        sort_field: typing.Optional[str] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        limit: typing.Optional[int] = None,
        page_token: typing.Optional[PageToken] = None,
        filters: typing.Optional[FilterQueryString] = None,
        request_options: typing.Optional[RequestOptions] = None
    ) -> AppointmentListPage:
        """
        Searches for appointments that match the query parameters.

        Parameters
        ----------
        sort_field : typing.Optional[str]
            The string path to the field to order by. Defaults to appointment.startTimestamp. Path values are camelCase.

        sort_direction : typing.Optional[SortDirection]
            Defaults to asc.

        limit : typing.Optional[int]
            Defaults to 100.

        page_token : typing.Optional[PageToken]

        filters : typing.Optional[FilterQueryString]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AppointmentListPage

        Examples
        --------
        from candid.client import CandidApiClient
        from candid.resources.pre_encounter import SortDirection

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.lists.v_1.get_appointment_list(
            sort_field="string",
            sort_direction=SortDirection.ASC,
            limit=1,
            page_token="string",
            filters="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "lists/v1/appointment",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "sort_field": sort_field,
                "sort_direction": sort_direction,
                "limit": limit,
                "page_token": page_token,
                "filters": filters,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(AppointmentListPage, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_patient_list(
        self,
        *,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        sort_field: typing.Optional[str] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        filters: typing.Optional[FilterQueryString] = None,
        request_options: typing.Optional[RequestOptions] = None
    ) -> PatientListPage:
        """
        Gets patients with dependent objects for patients that match the query parameters.

        Parameters
        ----------
        page_token : typing.Optional[PageToken]

        limit : typing.Optional[int]

        sort_field : typing.Optional[str]

        sort_direction : typing.Optional[SortDirection]
            Defaults to ascending.

        filters : typing.Optional[FilterQueryString]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatientListPage

        Examples
        --------
        import asyncio

        from candid.client import AsyncCandidApiClient
        from candid.resources.pre_encounter import SortDirection

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.lists.v_1.get_patient_list(
                page_token="string",
                limit=1,
                sort_field="string",
                sort_direction=SortDirection.ASC,
                filters="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "lists/v1/patient",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "page_token": page_token,
                "limit": limit,
                "sort_field": sort_field,
                "sort_direction": sort_direction,
                "filters": filters,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(PatientListPage, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequestError":
                raise BadRequestError(pydantic_v1.parse_obj_as(ErrorBase, _response_json["content"]))  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_appointment_list(
        self,
        *,
        sort_field: typing.Optional[str] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        limit: typing.Optional[int] = None,
        page_token: typing.Optional[PageToken] = None,
        filters: typing.Optional[FilterQueryString] = None,
        request_options: typing.Optional[RequestOptions] = None
    ) -> AppointmentListPage:
        """
        Searches for appointments that match the query parameters.

        Parameters
        ----------
        sort_field : typing.Optional[str]
            The string path to the field to order by. Defaults to appointment.startTimestamp. Path values are camelCase.

        sort_direction : typing.Optional[SortDirection]
            Defaults to asc.

        limit : typing.Optional[int]
            Defaults to 100.

        page_token : typing.Optional[PageToken]

        filters : typing.Optional[FilterQueryString]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AppointmentListPage

        Examples
        --------
        import asyncio

        from candid.client import AsyncCandidApiClient
        from candid.resources.pre_encounter import SortDirection

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.lists.v_1.get_appointment_list(
                sort_field="string",
                sort_direction=SortDirection.ASC,
                limit=1,
                page_token="string",
                filters="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "lists/v1/appointment",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "sort_field": sort_field,
                "sort_direction": sort_direction,
                "limit": limit,
                "page_token": page_token,
                "filters": filters,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(AppointmentListPage, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from .......core.api_error import ApiError
from .......core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .......core.datetime_utils import serialize_datetime
from .......core.jsonable_encoder import jsonable_encoder
from .......core.pydantic_utilities import pydantic_v1
from .......core.request_options import RequestOptions
from ....common.errors.not_found_error import NotFoundError
from ....common.errors.version_conflict_error import VersionConflictError
from ....common.types.not_found_error_body import NotFoundErrorBody
from ....common.types.page_token import PageToken
from ....common.types.patient_id import PatientId
from ....common.types.sort_direction import SortDirection
from ....common.types.version_conflict_error_body import VersionConflictErrorBody
from .errors.potential_duplicate_patients import PotentialDuplicatePatients
from .types.mutable_patient import MutablePatient
from .types.patient import Patient
from .types.patient_page import PatientPage
from .types.patient_sort_field import PatientSortField
from .types.potential_duplicate_patients_error_body import PotentialDuplicatePatientsErrorBody

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        request: MutablePatient,
        skip_duplicate_check: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Patient:
        """
        Adds a patient. VersionConflictError is returned when the patient's external ID is already in use.

        Parameters
        ----------
        request : MutablePatient

        skip_duplicate_check : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Patient

        Examples
        --------
        import datetime
        import uuid

        from candid.client import CandidApiClient
        from candid.resources.pre_encounter import (
            Address,
            AddressUse,
            ContactPoint,
            ContactPointUse,
            DisabilityStatus,
            Ethnicity,
            ExternalProvider,
            ExternalProviderType,
            Gender,
            HumanName,
            NameUse,
            Period,
            Race,
            Relationship,
            Sex,
            SexualOrientation,
        )
        from candid.resources.pre_encounter.patients.v_1 import (
            Contact,
            ExternalProvenance,
            FilingOrder,
            Guarantor,
            MaritalStatus,
            MutablePatient,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.patients.v_1.create(
            skip_duplicate_check=True,
            request=MutablePatient(
                name=HumanName(
                    family="string",
                    given=["string"],
                    use=NameUse.USUAL,
                    period=Period(
                        start=datetime.date.fromisoformat(
                            "2023-01-15",
                        ),
                        end=datetime.date.fromisoformat(
                            "2023-01-15",
                        ),
                    ),
                ),
                other_names=[
                    HumanName(
                        family="string",
                        given=["string"],
                        use=NameUse.USUAL,
                        period=Period(
                            start=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            end=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                        ),
                    )
                ],
                gender=Gender.MAN,
                birth_date=datetime.date.fromisoformat(
                    "2023-01-15",
                ),
                social_security_number="string",
                biological_sex=Sex.FEMALE,
                sexual_orientation=SexualOrientation.HETEROSEXUAL,
                race=Race.AMERICAN_INDIAN_OR_ALASKA_NATIVE,
                ethnicity=Ethnicity.HISPANIC_OR_LATINO,
                disability_status=DisabilityStatus.DISABLED,
                marital_status=MaritalStatus.ANNULLED,
                deceased=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
                multiple_birth=1,
                primary_address=Address(
                    use=AddressUse.HOME,
                    line=["string"],
                    city="string",
                    state="string",
                    postal_code="string",
                    country="string",
                    period=Period(
                        start=datetime.date.fromisoformat(
                            "2023-01-15",
                        ),
                        end=datetime.date.fromisoformat(
                            "2023-01-15",
                        ),
                    ),
                ),
                other_addresses=[
                    Address(
                        use=AddressUse.HOME,
                        line=["string"],
                        city="string",
                        state="string",
                        postal_code="string",
                        country="string",
                        period=Period(
                            start=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            end=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                        ),
                    )
                ],
                primary_telecom=ContactPoint(
                    value="string",
                    use=ContactPointUse.HOME,
                    period=Period(
                        start=datetime.date.fromisoformat(
                            "2023-01-15",
                        ),
                        end=datetime.date.fromisoformat(
                            "2023-01-15",
                        ),
                    ),
                ),
                other_telecoms=[
                    ContactPoint(
                        value="string",
                        use=ContactPointUse.HOME,
                        period=Period(
                            start=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            end=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                        ),
                    )
                ],
                email="string",
                electronic_communication_opt_in=True,
                photo="string",
                language="string",
                external_provenance=ExternalProvenance(
                    external_id="string",
                    system_name="string",
                ),
                contacts=[
                    Contact(
                        relationship=[Relationship.SELF],
                        name=HumanName(
                            family="string",
                            given=["string"],
                            use=NameUse.USUAL,
                            period=Period(
                                start=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                end=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                            ),
                        ),
                        telecoms=[
                            ContactPoint(
                                value="string",
                                use=ContactPointUse.HOME,
                                period=Period(
                                    start=datetime.date.fromisoformat(
                                        "2023-01-15",
                                    ),
                                    end=datetime.date.fromisoformat(
                                        "2023-01-15",
                                    ),
                                ),
                            )
                        ],
                        addresses=[
                            Address(
                                use=AddressUse.HOME,
                                line=["string"],
                                city="string",
                                state="string",
                                postal_code="string",
                                country="string",
                                period=Period(
                                    start=datetime.date.fromisoformat(
                                        "2023-01-15",
                                    ),
                                    end=datetime.date.fromisoformat(
                                        "2023-01-15",
                                    ),
                                ),
                            )
                        ],
                        period=Period(
                            start=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            end=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                        ),
                        hipaa_authorization=True,
                    )
                ],
                general_practitioners=[
                    ExternalProvider(
                        name=HumanName(
                            family="string",
                            given=["string"],
                            use=NameUse.USUAL,
                            period=Period(
                                start=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                end=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                            ),
                        ),
                        type=ExternalProviderType.PRIMARY,
                        npi="string",
                        telecoms=[
                            ContactPoint(
                                value="string",
                                use=ContactPointUse.HOME,
                                period=Period(
                                    start=datetime.date.fromisoformat(
                                        "2023-01-15",
                                    ),
                                    end=datetime.date.fromisoformat(
                                        "2023-01-15",
                                    ),
                                ),
                            )
                        ],
                        addresses=[{"key": "value"}],
                        period=Period(
                            start=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            end=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                        ),
                        canonical_id="string",
                    )
                ],
                filing_order=FilingOrder(
                    coverages=[
                        uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        )
                    ],
                ),
                non_insurance_payers=["string"],
                guarantor=Guarantor(
                    name=HumanName(
                        family="string",
                        given=["string"],
                        use=NameUse.USUAL,
                        period=Period(
                            start=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            end=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                        ),
                    ),
                    telecom=ContactPoint(
                        value="string",
                        use=ContactPointUse.HOME,
                        period=Period(
                            start=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            end=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                        ),
                    ),
                    email="string",
                    birth_date=datetime.date.fromisoformat(
                        "2023-01-15",
                    ),
                    address=Address(
                        use=AddressUse.HOME,
                        line=["string"],
                        city="string",
                        state="string",
                        postal_code="string",
                        country="string",
                        period=Period(
                            start=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            end=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                        ),
                    ),
                ),
                self_pay=True,
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "patients/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            params={"skip_duplicate_check": skip_duplicate_check},
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Patient, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    pydantic_v1.parse_obj_as(VersionConflictErrorBody, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "PotentialDuplicatePatients":
                raise PotentialDuplicatePatients(
                    pydantic_v1.parse_obj_as(PotentialDuplicatePatientsErrorBody, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        mrn: typing.Optional[str] = None,
        page_token: typing.Optional[PageToken] = None,
        sort_field: typing.Optional[PatientSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PatientPage:
        """
        Searches for patients that match the query parameters.

        Parameters
        ----------
        limit : typing.Optional[int]

        mrn : typing.Optional[str]

        page_token : typing.Optional[PageToken]

        sort_field : typing.Optional[PatientSortField]

        sort_direction : typing.Optional[SortDirection]
            Defaults to ascending.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatientPage

        Examples
        --------
        from candid.client import CandidApiClient
        from candid.resources.pre_encounter import SortDirection

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.patients.v_1.get_multi(
            limit=1,
            mrn="string",
            page_token="string",
            sort_field="string",
            sort_direction=SortDirection.ASC,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "patients/v1/get_multi",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "limit": limit,
                "mrn": mrn,
                "page_token": page_token,
                "sort_field": sort_field,
                "sort_direction": sort_direction,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(PatientPage, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, id: PatientId, *, request_options: typing.Optional[RequestOptions] = None) -> Patient:
        """
        Gets a patient.

        Parameters
        ----------
        id : PatientId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Patient

        Examples
        --------
        from candid.client import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.patients.v_1.get(
            id="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"patients/v1/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Patient, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    pydantic_v1.parse_obj_as(NotFoundErrorBody, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_history(
        self, id: PatientId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Patient]:
        """
        Gets a patient along with it's full history. The return list is ordered by version ascending.

        Parameters
        ----------
        id : PatientId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Patient]

        Examples
        --------
        from candid.client import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.patients.v_1.get_history(
            id="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"patients/v1/{jsonable_encoder(id)}/history",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(typing.List[Patient], _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    pydantic_v1.parse_obj_as(NotFoundErrorBody, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        id: PatientId,
        version: str,
        *,
        request: MutablePatient,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Patient:
        """
        Updates a patient. The path must contain the most recent version to prevent race conditions. Updating historic versions is not supported.

        Parameters
        ----------
        id : PatientId

        version : str

        request : MutablePatient

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Patient

        Examples
        --------
        import datetime
        import uuid

        from candid.client import CandidApiClient
        from candid.resources.pre_encounter import (
            Address,
            AddressUse,
            ContactPoint,
            ContactPointUse,
            DisabilityStatus,
            Ethnicity,
            ExternalProvider,
            ExternalProviderType,
            Gender,
            HumanName,
            NameUse,
            Period,
            Race,
            Relationship,
            Sex,
            SexualOrientation,
        )
        from candid.resources.pre_encounter.patients.v_1 import (
            Contact,
            ExternalProvenance,
            FilingOrder,
            Guarantor,
            MaritalStatus,
            MutablePatient,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.patients.v_1.update(
            id="string",
            version="string",
            request=MutablePatient(
                name=HumanName(
                    family="string",
                    given=["string"],
                    use=NameUse.USUAL,
                    period=Period(
                        start=datetime.date.fromisoformat(
                            "2023-01-15",
                        ),
                        end=datetime.date.fromisoformat(
                            "2023-01-15",
                        ),
                    ),
                ),
                other_names=[
                    HumanName(
                        family="string",
                        given=["string"],
                        use=NameUse.USUAL,
                        period=Period(
                            start=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            end=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                        ),
                    )
                ],
                gender=Gender.MAN,
                birth_date=datetime.date.fromisoformat(
                    "2023-01-15",
                ),
                social_security_number="string",
                biological_sex=Sex.FEMALE,
                sexual_orientation=SexualOrientation.HETEROSEXUAL,
                race=Race.AMERICAN_INDIAN_OR_ALASKA_NATIVE,
                ethnicity=Ethnicity.HISPANIC_OR_LATINO,
                disability_status=DisabilityStatus.DISABLED,
                marital_status=MaritalStatus.ANNULLED,
                deceased=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
                multiple_birth=1,
                primary_address=Address(
                    use=AddressUse.HOME,
                    line=["string"],
                    city="string",
                    state="string",
                    postal_code="string",
                    country="string",
                    period=Period(
                        start=datetime.date.fromisoformat(
                            "2023-01-15",
                        ),
                        end=datetime.date.fromisoformat(
                            "2023-01-15",
                        ),
                    ),
                ),
                other_addresses=[
                    Address(
                        use=AddressUse.HOME,
                        line=["string"],
                        city="string",
                        state="string",
                        postal_code="string",
                        country="string",
                        period=Period(
                            start=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            end=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                        ),
                    )
                ],
                primary_telecom=ContactPoint(
                    value="string",
                    use=ContactPointUse.HOME,
                    period=Period(
                        start=datetime.date.fromisoformat(
                            "2023-01-15",
                        ),
                        end=datetime.date.fromisoformat(
                            "2023-01-15",
                        ),
                    ),
                ),
                other_telecoms=[
                    ContactPoint(
                        value="string",
                        use=ContactPointUse.HOME,
                        period=Period(
                            start=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            end=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                        ),
                    )
                ],
                email="string",
                electronic_communication_opt_in=True,
                photo="string",
                language="string",
                external_provenance=ExternalProvenance(
                    external_id="string",
                    system_name="string",
                ),
                contacts=[
                    Contact(
                        relationship=[Relationship.SELF],
                        name=HumanName(
                            family="string",
                            given=["string"],
                            use=NameUse.USUAL,
                            period=Period(
                                start=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                end=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                            ),
                        ),
                        telecoms=[
                            ContactPoint(
                                value="string",
                                use=ContactPointUse.HOME,
                                period=Period(
                                    start=datetime.date.fromisoformat(
                                        "2023-01-15",
                                    ),
                                    end=datetime.date.fromisoformat(
                                        "2023-01-15",
                                    ),
                                ),
                            )
                        ],
                        addresses=[
                            Address(
                                use=AddressUse.HOME,
                                line=["string"],
                                city="string",
                                state="string",
                                postal_code="string",
                                country="string",
                                period=Period(
                                    start=datetime.date.fromisoformat(
                                        "2023-01-15",
                                    ),
                                    end=datetime.date.fromisoformat(
                                        "2023-01-15",
                                    ),
                                ),
                            )
                        ],
                        period=Period(
                            start=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            end=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                        ),
                        hipaa_authorization=True,
                    )
                ],
                general_practitioners=[
                    ExternalProvider(
                        name=HumanName(
                            family="string",
                            given=["string"],
                            use=NameUse.USUAL,
                            period=Period(
                                start=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                end=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                            ),
                        ),
                        type=ExternalProviderType.PRIMARY,
                        npi="string",
                        telecoms=[
                            ContactPoint(
                                value="string",
                                use=ContactPointUse.HOME,
                                period=Period(
                                    start=datetime.date.fromisoformat(
                                        "2023-01-15",
                                    ),
                                    end=datetime.date.fromisoformat(
                                        "2023-01-15",
                                    ),
                                ),
                            )
                        ],
                        addresses=[{"key": "value"}],
                        period=Period(
                            start=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            end=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                        ),
                        canonical_id="string",
                    )
                ],
                filing_order=FilingOrder(
                    coverages=[
                        uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        )
                    ],
                ),
                non_insurance_payers=["string"],
                guarantor=Guarantor(
                    name=HumanName(
                        family="string",
                        given=["string"],
                        use=NameUse.USUAL,
                        period=Period(
                            start=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            end=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                        ),
                    ),
                    telecom=ContactPoint(
                        value="string",
                        use=ContactPointUse.HOME,
                        period=Period(
                            start=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            end=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                        ),
                    ),
                    email="string",
                    birth_date=datetime.date.fromisoformat(
                        "2023-01-15",
                    ),
                    address=Address(
                        use=AddressUse.HOME,
                        line=["string"],
                        city="string",
                        state="string",
                        postal_code="string",
                        country="string",
                        period=Period(
                            start=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            end=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                        ),
                    ),
                ),
                self_pay=True,
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"patients/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Patient, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    pydantic_v1.parse_obj_as(NotFoundErrorBody, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    pydantic_v1.parse_obj_as(VersionConflictErrorBody, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def deactivate(
        self, id: PatientId, version: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Sets a patient as deactivated. The path must contain the most recent version to prevent race conditions. Deactivating historic versions is not supported. Subsequent updates via PUT to the patient will "reactivate" the patient and set the deactivated flag to false.

        Parameters
        ----------
        id : PatientId

        version : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from candid.client import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.patients.v_1.deactivate(
            id="string",
            version="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"patients/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    pydantic_v1.parse_obj_as(NotFoundErrorBody, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    pydantic_v1.parse_obj_as(VersionConflictErrorBody, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def search(
        self,
        *,
        mrn: typing.Optional[str] = None,
        similar_name_ordering: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Patient]:
        """
        Returns a list of Patients based on the search criteria.

        Parameters
        ----------
        mrn : typing.Optional[str]

        similar_name_ordering : typing.Optional[str]
            A string that is used to order similar names in search results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Patient]

        Examples
        --------
        from candid.client import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.patients.v_1.search(
            mrn="string",
            similar_name_ordering="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "patients/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={"mrn": mrn, "similar_name_ordering": similar_name_ordering},
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(typing.List[Patient], _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def scan(
        self, *, since: dt.datetime, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Patient]:
        """
        Scans up to 100 patient updates. The since query parameter is inclusive, and the result list is ordered by updatedAt ascending.

        Parameters
        ----------
        since : dt.datetime

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Patient]

        Examples
        --------
        import datetime

        from candid.client import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.patients.v_1.scan(
            since=datetime.datetime.fromisoformat(
                "2024-01-15 09:30:00+00:00",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "patients/v1/updates/scan",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={"since": serialize_datetime(since)},
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(typing.List[Patient], _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        request: MutablePatient,
        skip_duplicate_check: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Patient:
        """
        Adds a patient. VersionConflictError is returned when the patient's external ID is already in use.

        Parameters
        ----------
        request : MutablePatient

        skip_duplicate_check : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Patient

        Examples
        --------
        import asyncio
        import datetime
        import uuid

        from candid.client import AsyncCandidApiClient
        from candid.resources.pre_encounter import (
            Address,
            AddressUse,
            ContactPoint,
            ContactPointUse,
            DisabilityStatus,
            Ethnicity,
            ExternalProvider,
            ExternalProviderType,
            Gender,
            HumanName,
            NameUse,
            Period,
            Race,
            Relationship,
            Sex,
            SexualOrientation,
        )
        from candid.resources.pre_encounter.patients.v_1 import (
            Contact,
            ExternalProvenance,
            FilingOrder,
            Guarantor,
            MaritalStatus,
            MutablePatient,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.patients.v_1.create(
                skip_duplicate_check=True,
                request=MutablePatient(
                    name=HumanName(
                        family="string",
                        given=["string"],
                        use=NameUse.USUAL,
                        period=Period(
                            start=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            end=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                        ),
                    ),
                    other_names=[
                        HumanName(
                            family="string",
                            given=["string"],
                            use=NameUse.USUAL,
                            period=Period(
                                start=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                end=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                            ),
                        )
                    ],
                    gender=Gender.MAN,
                    birth_date=datetime.date.fromisoformat(
                        "2023-01-15",
                    ),
                    social_security_number="string",
                    biological_sex=Sex.FEMALE,
                    sexual_orientation=SexualOrientation.HETEROSEXUAL,
                    race=Race.AMERICAN_INDIAN_OR_ALASKA_NATIVE,
                    ethnicity=Ethnicity.HISPANIC_OR_LATINO,
                    disability_status=DisabilityStatus.DISABLED,
                    marital_status=MaritalStatus.ANNULLED,
                    deceased=datetime.datetime.fromisoformat(
                        "2024-01-15 09:30:00+00:00",
                    ),
                    multiple_birth=1,
                    primary_address=Address(
                        use=AddressUse.HOME,
                        line=["string"],
                        city="string",
                        state="string",
                        postal_code="string",
                        country="string",
                        period=Period(
                            start=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            end=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                        ),
                    ),
                    other_addresses=[
                        Address(
                            use=AddressUse.HOME,
                            line=["string"],
                            city="string",
                            state="string",
                            postal_code="string",
                            country="string",
                            period=Period(
                                start=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                end=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                            ),
                        )
                    ],
                    primary_telecom=ContactPoint(
                        value="string",
                        use=ContactPointUse.HOME,
                        period=Period(
                            start=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            end=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                        ),
                    ),
                    other_telecoms=[
                        ContactPoint(
                            value="string",
                            use=ContactPointUse.HOME,
                            period=Period(
                                start=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                end=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                            ),
                        )
                    ],
                    email="string",
                    electronic_communication_opt_in=True,
                    photo="string",
                    language="string",
                    external_provenance=ExternalProvenance(
                        external_id="string",
                        system_name="string",
                    ),
                    contacts=[
                        Contact(
                            relationship=[Relationship.SELF],
                            name=HumanName(
                                family="string",
                                given=["string"],
                                use=NameUse.USUAL,
                                period=Period(
                                    start=datetime.date.fromisoformat(
                                        "2023-01-15",
                                    ),
                                    end=datetime.date.fromisoformat(
                                        "2023-01-15",
                                    ),
                                ),
                            ),
                            telecoms=[
                                ContactPoint(
                                    value="string",
                                    use=ContactPointUse.HOME,
                                    period=Period(
                                        start=datetime.date.fromisoformat(
                                            "2023-01-15",
                                        ),
                                        end=datetime.date.fromisoformat(
                                            "2023-01-15",
                                        ),
                                    ),
                                )
                            ],
                            addresses=[
                                Address(
                                    use=AddressUse.HOME,
                                    line=["string"],
                                    city="string",
                                    state="string",
                                    postal_code="string",
                                    country="string",
                                    period=Period(
                                        start=datetime.date.fromisoformat(
                                            "2023-01-15",
                                        ),
                                        end=datetime.date.fromisoformat(
                                            "2023-01-15",
                                        ),
                                    ),
                                )
                            ],
                            period=Period(
                                start=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                end=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                            ),
                            hipaa_authorization=True,
                        )
                    ],
                    general_practitioners=[
                        ExternalProvider(
                            name=HumanName(
                                family="string",
                                given=["string"],
                                use=NameUse.USUAL,
                                period=Period(
                                    start=datetime.date.fromisoformat(
                                        "2023-01-15",
                                    ),
                                    end=datetime.date.fromisoformat(
                                        "2023-01-15",
                                    ),
                                ),
                            ),
                            type=ExternalProviderType.PRIMARY,
                            npi="string",
                            telecoms=[
                                ContactPoint(
                                    value="string",
                                    use=ContactPointUse.HOME,
                                    period=Period(
                                        start=datetime.date.fromisoformat(
                                            "2023-01-15",
                                        ),
                                        end=datetime.date.fromisoformat(
                                            "2023-01-15",
                                        ),
                                    ),
                                )
                            ],
                            addresses=[{"key": "value"}],
                            period=Period(
                                start=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                end=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                            ),
                            canonical_id="string",
                        )
                    ],
                    filing_order=FilingOrder(
                        coverages=[
                            uuid.UUID(
                                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                            )
                        ],
                    ),
                    non_insurance_payers=["string"],
                    guarantor=Guarantor(
                        name=HumanName(
                            family="string",
                            given=["string"],
                            use=NameUse.USUAL,
                            period=Period(
                                start=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                end=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                            ),
                        ),
                        telecom=ContactPoint(
                            value="string",
                            use=ContactPointUse.HOME,
                            period=Period(
                                start=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                end=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                            ),
                        ),
                        email="string",
                        birth_date=datetime.date.fromisoformat(
                            "2023-01-15",
                        ),
                        address=Address(
                            use=AddressUse.HOME,
                            line=["string"],
                            city="string",
                            state="string",
                            postal_code="string",
                            country="string",
                            period=Period(
                                start=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                end=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                            ),
                        ),
                    ),
                    self_pay=True,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "patients/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            params={"skip_duplicate_check": skip_duplicate_check},
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Patient, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    pydantic_v1.parse_obj_as(VersionConflictErrorBody, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "PotentialDuplicatePatients":
                raise PotentialDuplicatePatients(
                    pydantic_v1.parse_obj_as(PotentialDuplicatePatientsErrorBody, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        mrn: typing.Optional[str] = None,
        page_token: typing.Optional[PageToken] = None,
        sort_field: typing.Optional[PatientSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PatientPage:
        """
        Searches for patients that match the query parameters.

        Parameters
        ----------
        limit : typing.Optional[int]

        mrn : typing.Optional[str]

        page_token : typing.Optional[PageToken]

        sort_field : typing.Optional[PatientSortField]

        sort_direction : typing.Optional[SortDirection]
            Defaults to ascending.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatientPage

        Examples
        --------
        import asyncio

        from candid.client import AsyncCandidApiClient
        from candid.resources.pre_encounter import SortDirection

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.patients.v_1.get_multi(
                limit=1,
                mrn="string",
                page_token="string",
                sort_field="string",
                sort_direction=SortDirection.ASC,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "patients/v1/get_multi",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "limit": limit,
                "mrn": mrn,
                "page_token": page_token,
                "sort_field": sort_field,
                "sort_direction": sort_direction,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(PatientPage, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, id: PatientId, *, request_options: typing.Optional[RequestOptions] = None) -> Patient:
        """
        Gets a patient.

        Parameters
        ----------
        id : PatientId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Patient

        Examples
        --------
        import asyncio

        from candid.client import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.patients.v_1.get(
                id="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"patients/v1/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Patient, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    pydantic_v1.parse_obj_as(NotFoundErrorBody, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_history(
        self, id: PatientId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Patient]:
        """
        Gets a patient along with it's full history. The return list is ordered by version ascending.

        Parameters
        ----------
        id : PatientId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Patient]

        Examples
        --------
        import asyncio

        from candid.client import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.patients.v_1.get_history(
                id="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"patients/v1/{jsonable_encoder(id)}/history",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(typing.List[Patient], _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    pydantic_v1.parse_obj_as(NotFoundErrorBody, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        id: PatientId,
        version: str,
        *,
        request: MutablePatient,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Patient:
        """
        Updates a patient. The path must contain the most recent version to prevent race conditions. Updating historic versions is not supported.

        Parameters
        ----------
        id : PatientId

        version : str

        request : MutablePatient

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Patient

        Examples
        --------
        import asyncio
        import datetime
        import uuid

        from candid.client import AsyncCandidApiClient
        from candid.resources.pre_encounter import (
            Address,
            AddressUse,
            ContactPoint,
            ContactPointUse,
            DisabilityStatus,
            Ethnicity,
            ExternalProvider,
            ExternalProviderType,
            Gender,
            HumanName,
            NameUse,
            Period,
            Race,
            Relationship,
            Sex,
            SexualOrientation,
        )
        from candid.resources.pre_encounter.patients.v_1 import (
            Contact,
            ExternalProvenance,
            FilingOrder,
            Guarantor,
            MaritalStatus,
            MutablePatient,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.patients.v_1.update(
                id="string",
                version="string",
                request=MutablePatient(
                    name=HumanName(
                        family="string",
                        given=["string"],
                        use=NameUse.USUAL,
                        period=Period(
                            start=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            end=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                        ),
                    ),
                    other_names=[
                        HumanName(
                            family="string",
                            given=["string"],
                            use=NameUse.USUAL,
                            period=Period(
                                start=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                end=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                            ),
                        )
                    ],
                    gender=Gender.MAN,
                    birth_date=datetime.date.fromisoformat(
                        "2023-01-15",
                    ),
                    social_security_number="string",
                    biological_sex=Sex.FEMALE,
                    sexual_orientation=SexualOrientation.HETEROSEXUAL,
                    race=Race.AMERICAN_INDIAN_OR_ALASKA_NATIVE,
                    ethnicity=Ethnicity.HISPANIC_OR_LATINO,
                    disability_status=DisabilityStatus.DISABLED,
                    marital_status=MaritalStatus.ANNULLED,
                    deceased=datetime.datetime.fromisoformat(
                        "2024-01-15 09:30:00+00:00",
                    ),
                    multiple_birth=1,
                    primary_address=Address(
                        use=AddressUse.HOME,
                        line=["string"],
                        city="string",
                        state="string",
                        postal_code="string",
                        country="string",
                        period=Period(
                            start=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            end=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                        ),
                    ),
                    other_addresses=[
                        Address(
                            use=AddressUse.HOME,
                            line=["string"],
                            city="string",
                            state="string",
                            postal_code="string",
                            country="string",
                            period=Period(
                                start=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                end=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                            ),
                        )
                    ],
                    primary_telecom=ContactPoint(
                        value="string",
                        use=ContactPointUse.HOME,
                        period=Period(
                            start=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            end=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                        ),
                    ),
                    other_telecoms=[
                        ContactPoint(
                            value="string",
                            use=ContactPointUse.HOME,
                            period=Period(
                                start=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                end=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                            ),
                        )
                    ],
                    email="string",
                    electronic_communication_opt_in=True,
                    photo="string",
                    language="string",
                    external_provenance=ExternalProvenance(
                        external_id="string",
                        system_name="string",
                    ),
                    contacts=[
                        Contact(
                            relationship=[Relationship.SELF],
                            name=HumanName(
                                family="string",
                                given=["string"],
                                use=NameUse.USUAL,
                                period=Period(
                                    start=datetime.date.fromisoformat(
                                        "2023-01-15",
                                    ),
                                    end=datetime.date.fromisoformat(
                                        "2023-01-15",
                                    ),
                                ),
                            ),
                            telecoms=[
                                ContactPoint(
                                    value="string",
                                    use=ContactPointUse.HOME,
                                    period=Period(
                                        start=datetime.date.fromisoformat(
                                            "2023-01-15",
                                        ),
                                        end=datetime.date.fromisoformat(
                                            "2023-01-15",
                                        ),
                                    ),
                                )
                            ],
                            addresses=[
                                Address(
                                    use=AddressUse.HOME,
                                    line=["string"],
                                    city="string",
                                    state="string",
                                    postal_code="string",
                                    country="string",
                                    period=Period(
                                        start=datetime.date.fromisoformat(
                                            "2023-01-15",
                                        ),
                                        end=datetime.date.fromisoformat(
                                            "2023-01-15",
                                        ),
                                    ),
                                )
                            ],
                            period=Period(
                                start=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                end=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                            ),
                            hipaa_authorization=True,
                        )
                    ],
                    general_practitioners=[
                        ExternalProvider(
                            name=HumanName(
                                family="string",
                                given=["string"],
                                use=NameUse.USUAL,
                                period=Period(
                                    start=datetime.date.fromisoformat(
                                        "2023-01-15",
                                    ),
                                    end=datetime.date.fromisoformat(
                                        "2023-01-15",
                                    ),
                                ),
                            ),
                            type=ExternalProviderType.PRIMARY,
                            npi="string",
                            telecoms=[
                                ContactPoint(
                                    value="string",
                                    use=ContactPointUse.HOME,
                                    period=Period(
                                        start=datetime.date.fromisoformat(
                                            "2023-01-15",
                                        ),
                                        end=datetime.date.fromisoformat(
                                            "2023-01-15",
                                        ),
                                    ),
                                )
                            ],
                            addresses=[{"key": "value"}],
                            period=Period(
                                start=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                end=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                            ),
                            canonical_id="string",
                        )
                    ],
                    filing_order=FilingOrder(
                        coverages=[
                            uuid.UUID(
                                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                            )
                        ],
                    ),
                    non_insurance_payers=["string"],
                    guarantor=Guarantor(
                        name=HumanName(
                            family="string",
                            given=["string"],
                            use=NameUse.USUAL,
                            period=Period(
                                start=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                end=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                            ),
                        ),
                        telecom=ContactPoint(
                            value="string",
                            use=ContactPointUse.HOME,
                            period=Period(
                                start=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                end=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                            ),
                        ),
                        email="string",
                        birth_date=datetime.date.fromisoformat(
                            "2023-01-15",
                        ),
                        address=Address(
                            use=AddressUse.HOME,
                            line=["string"],
                            city="string",
                            state="string",
                            postal_code="string",
                            country="string",
                            period=Period(
                                start=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                end=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                            ),
                        ),
                    ),
                    self_pay=True,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"patients/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Patient, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    pydantic_v1.parse_obj_as(NotFoundErrorBody, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    pydantic_v1.parse_obj_as(VersionConflictErrorBody, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def deactivate(
        self, id: PatientId, version: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Sets a patient as deactivated. The path must contain the most recent version to prevent race conditions. Deactivating historic versions is not supported. Subsequent updates via PUT to the patient will "reactivate" the patient and set the deactivated flag to false.

        Parameters
        ----------
        id : PatientId

        version : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from candid.client import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.patients.v_1.deactivate(
                id="string",
                version="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"patients/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    pydantic_v1.parse_obj_as(NotFoundErrorBody, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    pydantic_v1.parse_obj_as(VersionConflictErrorBody, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def search(
        self,
        *,
        mrn: typing.Optional[str] = None,
        similar_name_ordering: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Patient]:
        """
        Returns a list of Patients based on the search criteria.

        Parameters
        ----------
        mrn : typing.Optional[str]

        similar_name_ordering : typing.Optional[str]
            A string that is used to order similar names in search results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Patient]

        Examples
        --------
        import asyncio

        from candid.client import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.patients.v_1.search(
                mrn="string",
                similar_name_ordering="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "patients/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={"mrn": mrn, "similar_name_ordering": similar_name_ordering},
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(typing.List[Patient], _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def scan(
        self, *, since: dt.datetime, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Patient]:
        """
        Scans up to 100 patient updates. The since query parameter is inclusive, and the result list is ordered by updatedAt ascending.

        Parameters
        ----------
        since : dt.datetime

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Patient]

        Examples
        --------
        import asyncio
        import datetime

        from candid.client import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.patients.v_1.scan(
                since=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "patients/v1/updates/scan",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={"since": serialize_datetime(since)},
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(typing.List[Patient], _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)
